\Lens.xpl
\Works like Polaris's Lens example in Hugi 31
\Compile with 32-bit XPL0 (xpx.bat)

include	c:\cxpl\codes;		\run-time library routines (intrinsics)

def	Radius = 150,		\lens radius (pixels)
	Radius2 = Radius*Radius,
	Zoom = 3;		\magnification factor

int	CpuReg;			\address of CPU register array (from GetReg)
char	Image(640*480);		\image from .bmp file



func	CallInt(Int, AX, BX, CX, DX, BP, DS, ES); \Call software interrupt
int	Int, AX, BX, CX, DX, BP, DS, ES; \(unused arguments need not be passed)
begin
CpuReg:= GetReg;
CpuReg(0):= AX;
CpuReg(1):= BX;
CpuReg(2):= CX;
CpuReg(3):= DX;
CpuReg(6):= BP;
CpuReg(9):= DS;
CpuReg(11):= ES;
SoftInt(Int);
return CpuReg(0) & $FFFF;	\return contents of AX register
end;	\CallInt

\------------------------------- MOUSE ROUTINES --------------------------------

func	OpenMouse;		\Initializes mouse; returns 'false' if it fails
begin				\Pointer is at center of screen but hidden
CallInt($21, $3533);		\Make sure vector ($33) points to something
if ((CpuReg(1) ! CpuReg(11)) & $FFFF) = 0 then return false;
return if CallInt($33, $0000) then true else false; \reset mouse & return status
end;	\OpenMouse



func	GetMousePosition(N); \Return position of specified mouse coordinate
int	N;	\0 = X coordinate; 1 = Y coordinate
\For video modes $0-$E and $13 the maximum coordinates are 639x199, minus
\ the size of the pointer. For modes $F-$12 the coordinates are the same as
\ the pixels. For 80-column text modes divide the mouse coordinates by 8 to
\ get the character cursor position.
begin
CallInt($33, $0003);
return (if N then CpuReg(3) else CpuReg(2)) & $0000FFFF;
end;	\GetMousePosition

\-------------------------------------------------------------------------------

proc	LoadBmp(Name);	\Load 640x480x8 .bmp file into Image and set color regs
char	Name;		\file name
int	Hand, I, T,
	X, Y,		\coordinates
	R, G, B;	\red, green, blue
begin
Hand:= FOpen(Name, 0);			\open file for input
FSet(Hand, ^I);				\set device 3 to handle
OpenI(3);

for I:= 0, 53 do T:= ChIn(3);		\skip unused header info

port($3C8):= 0;				\set color registers
for I:= 0, 255 do
	begin
	B:= ChIn(3)>>2;
	G:= ChIn(3)>>2;
	R:= ChIn(3)>>2;
	T:= ChIn(3);
	port($3C9):= R;
	port($3C9):= G;
	port($3C9):= B;
	end;

for Y:= -(480-1), 0 do			\.bmp files are upside down!
    for X:= 0, 640-1 do
	Image(X-Y*640):= ChIn(3);

FClose(Hand);
end;	\LoadBmp

\-------------------------------------------------------------------------------

proc	ExamineImage;	\Run magnifier until a key is struck
int	MX, MY,		\mouse coordinates
	X, Y,		\screen coordinates
	C;		\color of pixel to plot
begin
repeat	MX:= GetMousePosition(0);
	MY:= GetMousePosition(1);
	for Y:= 0, 480-1 do		\constantly scan entire Image
	    for X:= 0, 640-1 do
		begin
		if (MX-X)*(MX-X) + (MY-Y)*(MY-Y) <= Radius2 then \inside lens
		     C:= Image((X+(Zoom-1)*MX)/Zoom + (Y+(Zoom-1)*MY)/Zoom*640)
		else C:= Image(X+Y*640);
		Point(X, Y, C);
		end;
until	ChkKey;				\loop until a key is struck
end;	\ExamineImage

\-------------------------------------------------------------------------------

begin	\Main
if not OpenMouse then [Text(0, "A mouse is required");   exit];

SetVid($12);				\fool old mouse drivers
SetVid($101);				\640x480x8-bit color

LoadBmp("MRESCHER.BMP");
ExamineImage;

SetVid($03);				\restore normal text mode
end;	\Main
