/*
	Public Domain Code
	Coded by The IceBreaker
	
	Description: 	Make use of the small model loader library we implemented.
					This is a very simple program, so I am won't go into explanations
				 	because everything is self explanatory if you have a little
				 	background with OpenGL and WinAPI.
*/
#include "model3ds.h"
	
LRESULT CALLBACK WindowProc(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam);
	
nboolean handleMessages(MSG msg);	
void resizeGLWindow(int Width, int Height);		
void SetDCPixelFormat(HDC hDC);

int WINAPI WinMain	(	
						HINSTANCE hInst, HINSTANCE hPrevInst,
						LPTSTR szArgs, int iShowCmd
					)
{
	HWND 		hWnd;
	MSG  		msg;
	WNDCLASSEX 	wc;
	HDC hDC;
	HGLRC hRC;
	Model model;
	
	OPENFILENAME opf;
	char FileName[MAX_PATH] = "";
	
	wc.cbSize         = sizeof(WNDCLASSEX);
	wc.style          = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.cbClsExtra     = 0;
	wc.cbWndExtra     = 0;
	wc.hInstance      = hInst;
	wc.hIcon          = LoadIcon(hInst, MAKEINTRESOURCE(1));
	wc.hCursor        = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground  = (HBRUSH) GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName   = NULL;
	wc.lpszClassName  = "Loader3D";
	wc.hIconSm        = LoadIcon(hInst, MAKEINTRESOURCE(1));
	wc.lpfnWndProc    = (WNDPROC) WindowProc;

	RegisterClassEx(&wc);
	
	hWnd = CreateWindowEx	(	
								0, "Loader3D", "3DS Loader", WS_OVERLAPPEDWINDOW,
						 		30, 30, 640, 480, HWND_DESKTOP, NULL, hInst, NULL
							);
	
	ShowWindow(hWnd,SW_SHOW);
    UpdateWindow(hWnd);

	/* Get the filename */
	ZeroMemory(&opf,sizeof(opf));
	
	opf.lStructSize = sizeof(opf);
	opf.hwndOwner 	= hWnd;
	opf.lpstrFilter = "3DS Model Files (*.3ds)\0*.3ds\0";
    opf.lpstrFile 	= FileName; 
    opf.nMaxFile    = MAX_PATH;
	opf.lpstrTitle 	= "Open a 3DS Model File:";
    opf.Flags 		= OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
	
	if (GetOpenFileName(&opf))
	{
		/* load the model */
		model_Load(&model,FileName);
	} else { DestroyWindow(hWnd); return 13; }

	/* Init OpenGL */

	hDC = GetDC(hWnd);

	SetDCPixelFormat(hDC);

	hRC = wglCreateContext(hDC);
	wglMakeCurrent(hDC, hRC);

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepth(1.0f);

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	
	glDepthFunc(GL_LEQUAL);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	
	glShadeModel(GL_SMOOTH);
	
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

	resizeGLWindow(640,480); /* must be called at least once */

	while(handleMessages(msg))
	{
		/* Draw the scene here */
		
		glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
		glLoadIdentity();
		
		/* draw the model */
		model_Render(&model);
		/* rotate around Z axis */
		model_Rotate(&model,0.0f,0.0f,0.2f);
		
		SwapBuffers(hDC);
	}
	
	wglMakeCurrent(NULL, NULL);
	wglDeleteContext(hRC);
	
	ReleaseDC(hWnd, hDC);
	
	/* free the model */	
	model_Free(&model);
	
	return 666;	
}

LRESULT CALLBACK WindowProc(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch(Msg)
	{
		case WM_CLOSE:
			PostQuitMessage(0);
			break;
		case WM_SIZE:
			resizeGLWindow(LOWORD(lParam), HIWORD(lParam));	
			break;
        default:
            return DefWindowProc (hWnd, Msg, wParam, lParam);
	}
	
	return 0;	
}

nboolean handleMessages(MSG msg)
{
	while(PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{

			if (msg.message == WM_QUIT) 
			{ 
				return false;
			}

        	TranslateMessage(&msg);
	    	DispatchMessage(&msg);
		}
	return true;
}

void resizeGLWindow(int width, int height)
{
	if (height == 0) height=1;
    glViewport(0, 0, width, height);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    gluPerspective(45.0f, (float)width/(float)height, 0.1f, 500.0f);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void SetDCPixelFormat(HDC hDC)
{
	PIXELFORMATDESCRIPTOR pfd;
	int nPixelFormat;

	memset(&pfd,0,sizeof(pfd));

	pfd.nSize      = sizeof(pfd);
	pfd.nVersion   = 1;
	pfd.dwFlags    = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
	pfd.iPixelType = PFD_TYPE_RGBA;
	pfd.cColorBits = 16;
	pfd.cDepthBits = 32;
	pfd.iLayerType = PFD_MAIN_PLANE;

	nPixelFormat = ChoosePixelFormat(hDC, &pfd);

	SetPixelFormat(hDC, nPixelFormat, &pfd);
}
